'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _map = require('babel-runtime/core-js/map');

var _map2 = _interopRequireDefault(_map);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// https://github.com/w3c/IntersectionObserver/blob/main/polyfill/intersection-observer.js

var lazyLoad = function lazyLoad() {};

var CacheMap = new _map2.default();

lazyLoad.isSupportWebp = null;

lazyLoad.checkWebpSupport = function () {
  try {
    return document.createElement('canvas').toDataURL('image/webp').indexOf('data:image/webp') === 0;
  } catch (err) {
    return false;
  }
};

lazyLoad.loadNewImg = function (src) {
  return new _promise2.default(function (resolve, reject) {
    var img = new Image();
    img.onload = function () {
      resolve();
    };
    img.onerror = function () {
      reject();
    };
    img.src = src;
  });
};

lazyLoad.getUrl = function (binding) {
  if (this.options.openWebp && this.isSupportWebp) {
    lazyLoad.getUrl = function (binding) {
      return binding.value.webp || binding.value.image;
    };
  } else {
    lazyLoad.getUrl = function (binding) {
      return binding.value.image;
    };
  }
  return this.getUrl(binding);
};

// 加载图片
lazyLoad.load = function (info) {
  var _this = this;

  var el = info.el,
      binding = info.binding;

  var url = this.getUrl(binding);
  this.loadNewImg(url).then(function () {
    el.src = url;
    info.status = 'resolve';
    CacheMap.set(el, info);
  }).catch(function () {
    el.src = _this.options.failImg || '';
    info.status = 'reject';
    CacheMap.set(el, info);
  });
};

// 添加检测
lazyLoad.observer = function () {
  return new IntersectionObserver(function (entries) {
    entries.forEach(function (val) {
      if (val.isIntersecting) {
        var target = val.target;
        if (CacheMap.has(target)) {
          var info = CacheMap.get(target);
          if (info.status === 'pending') {
            lazyLoad.load(info);
          }
        }
      }
    });
  }, {
    rootMargin: '0px',
    threshold: 0
  });
};

// 检查图片是否被加载过
lazyLoad.checkComplete = function (url) {
  var img = new Image();
  img.src = url;

  return !!(img.complete || img.width);
};

lazyLoad.install = function (Vue, options) {
  var _this2 = this;

  // loading 时的图片
  // loading 失败的图片
  // webp true or false 默认 false
  var loadImg = options.loadImg,
      failImg = options.failImg,
      _options$openWebp = options.openWebp,
      openWebp = _options$openWebp === undefined ? true : _options$openWebp;


  this.options = {
    loadImg: loadImg,
    failImg: failImg,
    openWebp: openWebp
  };

  if (openWebp) {
    this.isSupportWebp = this.checkWebpSupport();
  }

  var observer = this.observer();

  Vue.directive('webp', {
    bind: function bind(el, binding) {

      var target = _this2.options.openWebp && _this2.isSupportWebp ? binding.value.webp : binding.value.image;

      var isLoaded = _this2.checkComplete(target);

      el.src = isLoaded ? target : loadImg;

      CacheMap.set(el, { el: el, binding: binding, status: isLoaded ? 'resolve' : 'pending' });

      console.log(CacheMap);

      observer.observe(el);
    },
    unbind: function unbind(el) {
      if (CacheMap.has(el)) {
        CacheMap.delete(el);
      }
    }
  });
};

exports.default = lazyLoad;